@@ -26,13 +26,12 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Any
 
 import torch
-from requests.exceptions import ConnectionError as RequestsConnectionError
-from transformers import AutoConfig, AutoTokenizer, PretrainedConfig
-
 from optimum.exporters.error_utils import AtolError, OutputMatchError, ShapeError
 from optimum.exporters.tasks import TasksManager
 from optimum.utils import is_diffusers_available, logging
 from optimum.utils.save_utils import maybe_load_preprocessors, maybe_save_preprocessors
+from requests.exceptions import ConnectionError as RequestsConnectionError
+from transformers import AutoConfig, AutoTokenizer, PretrainedConfig
 
 from ...neuron.models.auto_model import get_neuron_model_class, has_neuron_model_class
 from ...neuron.utils import (
@@ -44,6 +43,7 @@ from ...neuron.utils import (
     DIFFUSION_MODEL_UNET_NAME,
     DIFFUSION_MODEL_VAE_DECODER_NAME,
     DIFFUSION_MODEL_VAE_ENCODER_NAME,
+    DTYPE_MAPPER,
     ENCODER_NAME,
     NEURON_FILE_NAME,
     ImageEncoderArguments,
@@ -52,8 +52,9 @@ from ...neuron.utils import (
     LoRAAdapterArguments,
     is_neuron_available,
     is_neuronx_available,
-    map_torch_dtype,
 )
+from ...neuron.utils.instance import align_compilation_target
+from ...neuron.utils.system import get_available_cores
 from ...neuron.utils.version_utils import (
     check_compiler_compatibility_for_stable_diffusion,
 )
@@ -83,6 +84,7 @@ if is_neuronx_available():
 if is_diffusers_available():
     from diffusers import (
         DiffusionPipeline,
+        FluxKontextPipeline,
         FluxPipeline,
         ModelMixin,
         StableDiffusionPipeline,
@@ -98,10 +100,12 @@ logger.setLevel(logging.INFO)
 
 
 def infer_compiler_kwargs(args: argparse.Namespace) -> dict[str, Any]:
-    # infer compiler kwargs
+    instance_type = args.instance_type
     auto_cast = None if args.auto_cast == "none" else args.auto_cast
     auto_cast_type = None if auto_cast is None else args.auto_cast_type
-    compiler_kwargs = {"auto_cast": auto_cast, "auto_cast_type": auto_cast_type}
+    compiler_kwargs = {"auto_cast": auto_cast, "auto_cast_type": auto_cast_type, "instance_type": instance_type}
+
+    # Inf1 specific compiler args
     if hasattr(args, "disable_fast_relayout"):
         compiler_kwargs["disable_fast_relayout"] = getattr(args, "disable_fast_relayout")
     if hasattr(args, "disable_fallback"):
@@ -218,14 +222,14 @@ def normalize_diffusers_input_shapes(
 
 def infer_shapes_of_diffusers(
     input_shapes: dict[str, dict[str, int]],
-    model: "StableDiffusionPipeline | StableDiffusionXLPipeline | FluxPipeline",
+    model: "StableDiffusionPipeline | StableDiffusionXLPipeline | FluxPipeline | FluxKontextPipeline",
     has_controlnets: bool,
 ):
     max_sequence_length_1 = model.tokenizer.model_max_length if model.tokenizer is not None else None
     max_sequence_length_2 = (
         model.tokenizer_2.model_max_length if hasattr(model, "tokenizer_2") and model.tokenizer_2 is not None else None
     )
-    if isinstance(model, FluxPipeline):
+    if isinstance(model, (FluxPipeline, FluxKontextPipeline)):
         max_sequence_length_2 = input_shapes["text_encoder"].get("sequence_length", None) or max_sequence_length_2
 
     vae_encoder_num_channels = model.vae.config.in_channels
@@ -556,7 +560,7 @@ def load_models_and_neuron_configs(
     local_files_only: bool,
     token: bool | str | None,
     submodels: dict[str, Path | str] | None,
-    torch_dtype: str | torch.dtype | None = None,
+    torch_dtype: str | torch.dtype = torch.float32,
     tensor_parallel_size: int = 1,
     controlnet_ids: str | list[str] | None = None,
     lora_args: LoRAAdapterArguments | None = None,
@@ -615,7 +619,7 @@ def main_export(
     model_name_or_path: str,
     output: str | Path,
     compiler_kwargs: dict[str, Any],
-    torch_dtype: str | torch.dtype | None = None,
+    torch_dtype: str | torch.dtype = torch.float32,
     tensor_parallel_size: int = 1,
     model: "PreTrainedModel | ModelMixin | None" = None,
     task: str = "auto",
@@ -643,7 +647,7 @@ def main_export(
     **input_shapes,
 ):
     output = Path(output)
-    torch_dtype = map_torch_dtype(torch_dtype)
+    torch_dtype = DTYPE_MAPPER.pt(torch_dtype)
     if not output.parent.exists():
         output.parent.mkdir(parents=True)
 
@@ -692,12 +696,17 @@ def main_export(
     )
 
     # Validate compiled model
-    if do_validation and tensor_parallel_size > 1:
-        # TODO: support the validation of tp models.
-        logger.warning(
-            "The validation is not yet supported for tensor parallel model, the validation will be turned off."
-        )
-        do_validation = False
+    if do_validation:
+        if tensor_parallel_size > 1:
+            # TODO: support the validation of tp models.
+            logger.warning(
+                "The validation is not yet supported for tensor parallel model, the validation will be turned off."
+            )
+            do_validation = False
+        elif get_available_cores() == 0:
+            logger.info("The validation is disabled since no Neuron device is detected.")
+            do_validation = False
+
     if do_validation is True:
         try:
             validate_models_outputs(
@@ -731,20 +740,26 @@ def main_export(
 
 def maybe_export_from_neuron_model_class(
     model: str,
+    config: PretrainedConfig,
     output: str | Path,
-    task: str = "auto",
+    task: str,
+    instance_type,
     cache_dir: str | None = None,
     subfolder: str = "",
-    trust_remote_code: bool = False,
+    token: str | None = None,
+    revision: str | None = None,
+    trust_remote_code: bool | None = None,
+    batch_size: int | None = None,
+    sequence_length: int | None = None,
+    tensor_parallel_size: int | None = None,
     **kwargs,
 ):
     """Export the model from the neuron model class if it exists."""
-    if task == "auto":
-        task = infer_task(model)
     output = Path(output)
     # Remove None values from the kwargs
     kwargs = {key: value for key, value in kwargs.items() if value is not None}
     # Also remove some arguments that are not supported in this context
+    kwargs.pop("auto_cast_type", None)
     kwargs.pop("disable_neuron_cache", None)
     kwargs.pop("inline_weights_neff", None)
     kwargs.pop("O1", None)
@@ -754,21 +769,27 @@ def maybe_export_from_neuron_model_class(
     kwargs.pop("dynamic_batch_size", None)
     kwargs.pop("output_hidden_states", None)
     kwargs.pop("output_attentions", None)
-    kwargs.pop("tensor_parallel_size", None)
-    # Fetch the model config
-    config = AutoConfig.from_pretrained(model)
     # Check if we have an auto-model class for the model_type and task
     if not has_neuron_model_class(model_type=config.model_type, task=task, mode="inference"):
         return False
     neuron_model_class = get_neuron_model_class(model_type=config.model_type, task=task, mode="inference")
-    neuron_model = neuron_model_class.from_pretrained(
+    neuron_config = neuron_model_class.get_neuron_config(
+        model_name_or_path=model,
+        config=config,
+        token=token,
+        revision=revision,
+        instance_type=instance_type,
+        batch_size=batch_size,
+        sequence_length=sequence_length,
+        tensor_parallel_size=tensor_parallel_size,
+    )
+    neuron_model = neuron_model_class.export(
         model_id=model,
-        export=True,
+        config=config,
+        neuron_config=neuron_config,
         cache_dir=cache_dir,
         subfolder=subfolder,
-        config=config,
         trust_remote_code=trust_remote_code,
-        load_weights=False,  # Reduce model size for nxd models
         **kwargs,
     )
     if not output.parent.exists():
@@ -790,7 +811,39 @@ def main():
     # Retrieve CLI arguments
     args = parser.parse_args()
 
+    if args.instance_type is not None:
+        # We must align the compilation target before neuronx-distributed is initialized
+        align_compilation_target(args.instance_type, override=True)
+
     task = infer_task(args.model) if args.task == "auto" else args.task
+
+    try:
+        # Try first the export based on custom modeling classes
+        config = AutoConfig.from_pretrained(args.model)
+        if task == "text-generation":
+            # In case a multi-modal model is being exported, extract the text model config
+            config = config.get_text_config()
+    except Exception:
+        config = None
+    if config is not None:
+        kwargs = vars(args).copy()
+        kwargs.pop("task")
+        if maybe_export_from_neuron_model_class(
+            model=kwargs.pop("model"),
+            config=config,
+            output=kwargs.pop("output"),
+            task=task,
+            instance_type=kwargs.pop("instance_type"),
+            cache_dir=kwargs.pop("cache_dir", None),
+            subfolder=kwargs.pop("subfolder", None),
+            token=kwargs.pop("token", None),
+            revision=kwargs.pop("revision", None),
+            trust_remote_code=kwargs.pop("trust_remote_code", None),
+            **kwargs,
+        ):
+            return
+        logger.info(f"No custom modeling class is registered for {args.model} with task {task}")
+
     library_name = TasksManager.infer_library_from_model(args.model, cache_dir=args.cache_dir)
 
     if library_name == "diffusers":
@@ -800,11 +853,6 @@ def main():
         input_shapes = normalize_sentence_transformers_input_shapes(args)
         submodels = None
     else:
-        # New export mode using dedicated neuron model classes
-        kwargs = vars(args).copy()
-        if maybe_export_from_neuron_model_class(**kwargs):
-            return
-        # Fallback to legacy export
         input_shapes = get_input_shapes(task, args)
         submodels = None
 
@@ -812,6 +860,7 @@ def main():
     compiler_kwargs = infer_compiler_kwargs(args)
     optional_outputs = customize_optional_outputs(args)
     optlevel = parse_optlevel(args)
+
     lora_args = LoRAAdapterArguments(
         model_ids=getattr(args, "lora_model_ids", None),
         weight_names=getattr(args, "lora_weight_names", None),
