@@ -32,36 +32,36 @@ from accelerate.utils.operations import gather_object, recursively_apply
 from neuronx_distributed import parallel_layers
 from neuronx_distributed.optimizer import NeuronZero1Optimizer
 from neuronx_distributed.parallel_layers.parallel_state import (
-    get_data_parallel_group,
-    get_pipeline_model_parallel_size,
-    get_tensor_model_parallel_group,
-    model_parallel_is_initialized,
+    get_context_model_parallel_size,
+    get_data_parallel_replica_groups,
+    get_data_parallel_size,
+    get_tensor_model_parallel_replica_groups,
 )
 from neuronx_distributed.utils.model_utils import move_model_to_device
-from torch.utils.data import DataLoader
+from torch.utils.data import DataLoader, IterableDataset
 from torch.utils.data.distributed import DistributedSampler
 from torch_xla.distributed.parallel_loader import MpDeviceLoader
 from transformers import PreTrainedModel
+from transformers.training_args import trainer_log_levels
 
 from ...utils import logging
 from ..models.neuron_config import TrainingNeuronConfig
 from ..models.training.pipeline_utils import create_nxdpp_model
+from ..models.training.training_utils import is_custom_modeling_model, is_logging_process
 from ..utils import (
     DynamicPatch,
     ModelPatcher,
     Patcher,
     patch_within_function,
 )
-from ..utils.misc import args_and_kwargs_to_kwargs_only, is_main_worker
-from ..utils.training_utils import is_custom_modeling_model
+from ..utils.misc import args_and_kwargs_to_kwargs_only
 from .optimizer import NeuronAcceleratedOptimizer
 from .scheduler import NeuronAcceleratedScheduler
 from .state import NeuronAcceleratorState
 from .utils import (
-    AutocastBackend,
-    NeuronDistributedType,
     patch_accelerate_is_torch_xla_available,
 )
+from .utils.dataclasses import MixedPrecisionConfig, MixedPrecisionMode
 from .utils.misc import (
     apply_activation_checkpointing,
     create_patched_save_pretrained,
@@ -69,7 +69,11 @@ from .utils.misc import (
 from .utils.operations import _xla_gather
 
 
+# Setup logging so that the main process logs at the INFO level and the others are silent.
+log_levels = dict(**trainer_log_levels, silent=100)
 logger = logging.get_logger(__name__)
+log_level = "info" if is_logging_process() else "silent"
+logging.set_verbosity(log_levels[log_level])
 
 
 class NeuronAccelerator(Accelerator):
@@ -78,7 +82,7 @@ class NeuronAccelerator(Accelerator):
         *args,
         trn_config: TrainingNeuronConfig | None = None,
         zero_1: bool = False,
-        autocast_backend: str | AutocastBackend = "xla",
+        mixed_precision_config: MixedPrecisionConfig | str | None = None,
         **kwargs,
     ):
         # Patches accelerate.utils.imports.is_tpu_available to match `is_torch_xla_available`
@@ -139,33 +143,27 @@ class NeuronAccelerator(Accelerator):
         full_kwargs["gradient_accumulation_plugin"] = gradient_accumulation_plugin
         full_kwargs["gradient_accumulation_steps"] = gradient_accumulation_steps
 
-        if not isinstance(autocast_backend, AutocastBackend):
-            autocast_backend = AutocastBackend(autocast_backend)
-
-        # TODO: remove this, it should already be patched.
-        # The original `is_torch_xla_available` function is checking for TPU or GPU in `accelerate`.
-        # Here, we patch it to return True for Neuron cores as well.
-        # def patched_is_torch_xla_available(check_is_tpu: bool = False, check_is_gpu: bool = False) -> bool:
-        #     return True
-
-        # import accelerate
-
-        # accelerate.state.is_torch_xla_available = patched_is_torch_xla_available
-
-        patched_accelerator_state = partial(
-            NeuronAcceleratorState, trn_config=trn_config, autocast_backend=autocast_backend
+        if isinstance(mixed_precision_config, str):
+            mixed_precision_config = MixedPrecisionConfig(mixed_precision_config)
+        elif mixed_precision_config is None:
+            mixed_precision_config = MixedPrecisionConfig("NO")
+        self.mixed_precision_config = mixed_precision_config
+        full_kwargs["mixed_precision"] = (
+            "bf16" if self.mixed_precision_config.mode is MixedPrecisionMode.AUTOCAST_BF16 else "no"
         )
+
+        patched_accelerator_state = partial(NeuronAcceleratorState, trn_config=trn_config)
         with Patcher([("accelerate.accelerator.AcceleratorState", patched_accelerator_state)]):
             super().__init__(**full_kwargs)
 
         self.zero_1 = zero_1
 
         if self.autocast_handler is None:
-            enabled = self.state.mixed_precision == "bf16" and autocast_backend is AutocastBackend.AMP
+            enabled = self.state.mixed_precision == "bf16"
             self.autocast_handler = AutocastKwargs(enabled=enabled)
 
         if self.process_index == -1 and self.zero_1:
-            raise ValueError("XLA ZeRO Stage 1 can only be enabled in a distributed training setting.")
+            raise ValueError("ZeRO-1 can only be enabled in a distributed training setting.")
 
         if num_steps != 1:
             self.gradient_accumulation_steps = num_steps
@@ -205,7 +203,6 @@ class NeuronAccelerator(Accelerator):
             drop_last=data_loader.drop_last or force_drop_last,
         )
 
-        distributed_dataloader._is_accelerate_prepared = True
         return distributed_dataloader
 
     def prepare_data_loader(
@@ -214,56 +211,90 @@ class NeuronAccelerator(Accelerator):
         device_placement: bool | None = None,
         slice_fn_for_dispatch: Callable | None = None,
         use_mp_device_loader: bool = False,
+        batches_per_execution: int = 1,
     ):
         if slice_fn_for_dispatch is not None:
             raise NotImplementedError(
                 "The `slice_fn_for_dispatch` argument is not supported in `NeuronAccelerator.prepare_data_loader`."
             )
         force_drop_last = False
-        if self.state.distributed_type is NeuronDistributedType.MODEL_PARALLELISM:
+        if self.state.trn_config.model_parallelism_enabled:
             num_replicas = parallel_layers.parallel_state.get_data_parallel_size()
             rank = parallel_layers.parallel_state.get_data_parallel_rank()
             force_drop_last = parallel_layers.parallel_state.get_pipeline_model_parallel_size() > 1
-            if is_main_worker() and force_drop_last:
-                logger.warning(
-                    "Pipeline parallelsim: forcing the dataloader to drop the last incomplete batch because it can "
-                    "cause failure if the last batch size is not divisible by the number of microbatches for the pipeline."
-                )
+            logger.warning(
+                "Pipeline parallelsim: forcing the dataloader to drop the last incomplete batch because it can "
+                "cause failure if the last batch size is not divisible by the number of microbatches for the pipeline."
+            )
         else:
             num_replicas = xr.world_size()
             rank = xr.global_ordinal()
         if self.state.num_processes > 1:
-            data_loader = self._prepare_data_loader_for_distributed(
-                data_loader, num_replicas=num_replicas, rank=rank, force_drop_last=force_drop_last
-            )
+            if isinstance(data_loader.dataset, IterableDataset):
+                logger.warning(
+                    "Using an IterableDataset with multiple processes. Make sure that each process loads the correct data."
+                )
+            if not isinstance(data_loader.dataset, IterableDataset):
+                data_loader = self._prepare_data_loader_for_distributed(
+                    data_loader, num_replicas=num_replicas, rank=rank, force_drop_last=force_drop_last
+                )
             # No need to wrap the dataloader if we are using pipeline parallelism.
             if use_mp_device_loader and self.state.trn_config.pipeline_parallel_size == 1:
-                data_loader = MpDeviceLoader(data_loader, self.device)
+                data_loader = MpDeviceLoader(
+                    data_loader,
+                    self.device,
+                    batches_per_execution=batches_per_execution,
+                    loader_prefetch_size=2 * batches_per_execution,
+                    device_prefetch_size=batches_per_execution,
+                )
+        data_loader._is_accelerate_prepared = True
         return data_loader
 
     def _prepare_optimizer_for_zero_1(self, optimizer: torch.optim.Optimizer, device_placement=None):
-        mixed_precision_to_dtype = {
-            "no": torch.float32,
-            "bf16": torch.bfloat16,
-        }
-        optimizer_dtype = mixed_precision_to_dtype.get(self.state.mixed_precision, None)
-        if optimizer_dtype is None:
-            raise ValueError(f"The precision {self.state.mixed_precision} is not supported for ZeRO Stage 1")
+        mixed_precision_config = self.mixed_precision_config
 
-        if not model_parallel_is_initialized():
-            sharding_groups = None
-            grad_norm_groups = None
+        context_model_size = get_context_model_parallel_size()
+        if context_model_size > 1:
+            raise RuntimeError(
+                "Context model parallelism is not supported with ZeRO-1. Please set `trn_config.context_model_parallel_size=1`."
+            )
+        sharding_groups = get_data_parallel_replica_groups()
+        grad_norm_groups = get_tensor_model_parallel_replica_groups()
+
+        zero_1_config = {
+            "pin_layout": False,
+            "sharding_groups": sharding_groups,
+            "grad_norm_groups": grad_norm_groups,
+        }
+
+        # P148368176: Bucket cap >140MB causes NaN at step 3 (known issue)
+        _ALL_GATHER_REDUCE_SCATTER_BUCKET_CAP_MB = 130
+        bucket_cap = int(
+            os.getenv("ALL_GATHER_REDUCE_SCATTER_BUCKET_CAP_MB", _ALL_GATHER_REDUCE_SCATTER_BUCKET_CAP_MB)
+        )
+        reduce_scatter_bucket_cap = bucket_cap
+        all_gather_bucket_cap = max(1, bucket_cap // get_data_parallel_size())
+        zero_1_config["bucket_cap_mb_all_gather"] = all_gather_bucket_cap
+        zero_1_config["bucket_cap_mb_reduce_scatter"] = reduce_scatter_bucket_cap
+
+        if mixed_precision_config.optimizer_use_master_weights:
+            zero_1_config["optimizer_dtype"] = torch.float32
         else:
-            sharding_groups = get_data_parallel_group(as_list=True)
-            grad_norm_groups = get_tensor_model_parallel_group(as_list=True)
+            zero_1_config["optimizer_dtype"] = torch.bfloat16
+
+        # use_fp32_grad_acc cannot be True if use_master_weights is False
+        # It is already checked in MixedPrecisionConfig
+        zero_1_config["use_grad_acc_hook"] = mixed_precision_config.optimizer_use_fp32_grad_acc
+        zero_1_config["higher_cc_precision"] = mixed_precision_config.optimizer_use_fp32_grad_acc
+
+        # save_master_weights_in_ckpt cannot be True if use_master_weights is False
+        # It is already checked in MixedPrecisionConfig
+        zero_1_config["save_master_weights"] = mixed_precision_config.optimizer_save_master_weights_in_ckpt
 
         zero_1_optimizer = NeuronZero1Optimizer(
             optimizer.param_groups,
             optimizer.__class__,
-            optimizer_dtype=optimizer_dtype,
-            pin_layout=False,
-            sharding_groups=sharding_groups,
-            grad_norm_groups=grad_norm_groups,
+            **zero_1_config,
         )
 
         return zero_1_optimizer
@@ -305,13 +336,16 @@ class NeuronAccelerator(Accelerator):
         return model
 
     def prepare_model(
-        self, model: torch.nn.Module, device_placement: bool | None = None, evaluation_mode: bool = False
+        self,
+        model: torch.nn.Module,
+        device_placement: bool | None = None,
+        evaluation_mode: bool = False,
     ):
         # If the model was already prepared, we skip.
         if model in self._models:
             return model
 
-        if self.distributed_type is NeuronDistributedType.MODEL_PARALLELISM and not is_custom_modeling_model(model):
+        if self.state.trn_config.model_parallelism_enabled and not is_custom_modeling_model(model):
             raise NotImplementedError(
                 "Model parallelism is only supported for models with a custom modeling implementation."
             )
@@ -324,19 +358,20 @@ class NeuronAccelerator(Accelerator):
         model.config.output_attentions = False
         model.config.output_hidden_states = False
 
+        full_bf16 = self.mixed_precision_config.mode is MixedPrecisionMode.FULL_BF16
+
         if is_custom_modeling_model(model):
-            if get_pipeline_model_parallel_size() > 1:
+            if self.state.trn_config.pipeline_parallel_size > 1:
+                if full_bf16:
+                    model.to(torch.bfloat16)
                 model = create_nxdpp_model(model)
                 model.move_model_to_device()
             else:
+                if full_bf16:
+                    model.to(torch.bfloat16)
                 move_model_to_device(model, self.device)
                 model.tie_weights()
-            model = super().prepare_model(model, device_placement=False, evaluation_mode=evaluation_mode)
         else:
-            # Question: should do the same for custom models?
-            if self.state.mixed_precision == "bf16":
-                model.to(torch.bfloat16)
-
             should_apply_activation_checkpointing = False
             for mod in model.modules():
                 if getattr(mod, "gradient_checkpointing", False):
@@ -347,17 +382,19 @@ class NeuronAccelerator(Accelerator):
             for module in model.modules():
                 if getattr(module, "_use_sdpa", False):
                     module._use_sdpa = False
-                if getattr(module, "_use_flash_attention_2", False):
-                    module._use_flash_attention_2 = False
 
             if should_apply_activation_checkpointing:
                 apply_activation_checkpointing(model)
+            if full_bf16:
+                model.to(torch.bfloat16)
             move_model_to_device(model, xm.xla_device())
             model.tie_weights()
-            device_placement = False
-            model = super().prepare_model(model, device_placement=device_placement, evaluation_mode=evaluation_mode)
 
         xm.mark_step()
+
+        # Adding the model to the list of prepared models.
+        self._models.append(model)
+
         return model
 
     def backward(self, loss, **kwargs):
@@ -371,14 +408,12 @@ class NeuronAccelerator(Accelerator):
     @contextlib.contextmanager
     def autocast(self, autocast_handler: AutocastKwargs | None = None):
         if autocast_handler is None:
-            # By default `self.autocast_handler` enables autocast if:
-            #   - `self.state.mixed_precision == "bf16"`
-            #   - `self.state.autocast_backend is AutocastBackend.AMP`
+            # By default `self.autocast_handler` enables autocast if `self.state.mixed_precision == "bf16"`
             autocast_handler = self.autocast_handler
 
         if autocast_handler.enabled:
             autocast_kwargs = autocast_handler.to_kwargs()
-            autocast_context = torch.autocast(dtype=torch.bfloat16, device_type="cuda", **autocast_kwargs)
+            autocast_context = torch.autocast(dtype=torch.bfloat16, device_type="xla", **autocast_kwargs)
         else:
             autocast_context = contextlib.nullcontext()
 
@@ -506,7 +541,7 @@ class NeuronAccelerator(Accelerator):
     def save_state(
         self, output_dir: str | None = None, safe_serialization: bool = True, **save_model_func_kwargs
     ) -> str:
-        if self.distributed_type is NeuronDistributedType.MODEL_PARALLELISM:
+        if self.state.trn_config.model_parallelism_enabled:
             return self.save_state_for_mp(output_dir=output_dir, **save_model_func_kwargs)
         return super().save_state(
             output_dir=output_dir, safe_serialization=safe_serialization, **save_model_func_kwargs
@@ -554,3 +589,8 @@ class NeuronAccelerator(Accelerator):
         except Exception:
             # Dataset had no length or raised an error
             return data
+
+    @contextlib.contextmanager
+    def accumulate(self):
+        self._do_sync()
+        yield
