@@ -12,28 +12,32 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from typing import TYPE_CHECKING
+from abc import ABC
+from dataclasses import dataclass
 
 import torch
 
-from optimum.modeling_base import OptimizedModel
+
+class PreTrainedModel:
+    # A fake PreTrainedModel class to be inserted in NeuronModel class hierarchy
+    # to fool transformers pipeline framework identification algorithm
+    pass
 
 
-if TYPE_CHECKING:
-    from transformers import PretrainedConfig, PreTrainedModel
+@dataclass
+class NeuronModel(PreTrainedModel, ABC):
+    """Base class for all Neuron models."""
 
+    device: torch.device = torch.device("cpu")
 
-class NeuronModel(OptimizedModel):
-    def __init__(self, model: "PreTrainedModel", config: "PretrainedConfig"):
-        super().__init__(model, config)
-        if hasattr(model, "device"):
-            self.device = model.device
-        else:
-            self.device = torch.device("cpu")
+    def __call__(self, *args, **kwargs):
+        return self.forward(*args, **kwargs)
 
     def to(self, device: str | torch.device):
         if not isinstance(device, torch.device):
             device = torch.device(device)
         if device.type != self.device.type:
             raise ValueError(f"Neuron models cannot be moved to {device.type}.")
+
+    def forward(self, *args, **kwargs):
+        raise NotImplementedError("NeuronModel is an abstract class. Please use a subclass.")
