@@ -21,6 +21,7 @@ import os
 import shutil
 from abc import abstractmethod
 from collections import OrderedDict
+from contextlib import contextmanager
 from dataclasses import asdict
 from pathlib import Path
 from tempfile import TemporaryDirectory
@@ -30,6 +31,8 @@ import neuronx_distributed
 import torch
 import torch_neuronx
 from huggingface_hub import snapshot_download
+from optimum.exporters.tasks import TasksManager
+from optimum.utils import is_diffusers_available, logging
 from safetensors.torch import load_file
 from torch.nn import ModuleList
 from transformers import CLIPFeatureExtractor, CLIPTokenizer, PretrainedConfig, T5Tokenizer
@@ -42,8 +45,6 @@ from optimum.exporters.neuron import (
     replace_stable_diffusion_submodels,
 )
 from optimum.exporters.neuron.model_configs import *  # noqa: F403
-from optimum.exporters.tasks import TasksManager
-from optimum.utils import is_diffusers_available, logging
 
 from .cache.entries.multi_model import MultiModelCacheEntry
 from .cache.hub_cache import create_hub_compile_cache_proxy
@@ -64,7 +65,9 @@ from .utils import (
     replace_weights,
     store_compilation_config,
 )
+from .utils.instance import align_compilation_target, get_default_compilation_target
 from .utils.require_utils import requires_torch_neuronx
+from .utils.system import get_neuron_major
 from .utils.version_utils import get_neuronxcc_version
 
 
@@ -74,6 +77,8 @@ NEURON_COMPILER_VERSION = get_neuronxcc_version()
 if is_diffusers_available():
     from diffusers import (
         ControlNetModel,
+        FluxInpaintPipeline,
+        FluxKontextPipeline,
         FluxPipeline,
         LatentConsistencyModelPipeline,
         LCMScheduler,
@@ -646,7 +651,7 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
                 revision=revision,
                 force_download=force_download,
                 allow_patterns=allow_patterns,
-                ignore_patterns=["*.msgpack", "*.safetensors", "*.bin"],
+                ignore_patterns=["*.msgpack", "*.bin"],
             )
 
         new_model_save_dir = Path(model_id)
@@ -776,7 +781,7 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
         cls,
         model_id: str | Path,
         config: dict[str, Any],
-        torch_dtype: str | torch.dtype | None = None,
+        torch_dtype: str | torch.dtype = torch.float32,
         unet_id: str | Path | None = None,
         token: bool | str | None = None,
         revision: str = "main",
@@ -786,6 +791,7 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
         tensor_parallel_size: int | None = 1,
         disable_neuron_cache: bool = False,
         inline_weights_to_neff: bool = True,
+        instance_type: Literal["trn1", "inf2", "trn1n", "trn2"] | None = None,
         optlevel: str = "2",
         subfolder: str = "",
         local_files_only: bool = False,
@@ -833,11 +839,13 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
                 Whether to disable automatic caching of compiled models. If set to True, will not load neuron cache nor cache the compiled artifacts.
             inline_weights_to_neff (`bool`, defaults to `True`):
                 Whether to inline the weights to the neff graph. If set to False, weights will be separated from the neff.
+            instance_type (`Literal["trn1", "inf2", "trn1n", "trn2"] | None`, defaluts to `None`):
+                Target Neuron instance type on which the compiled model will be run, valid values are: "trn1", "inf2", "trn1n", "trn2".
             optlevel (`str`, defaults to `"2"`):
-            The level of optimization the compiler should perform. Can be `"1"`, `"2"` or `"3"`, defaults to "2".
-                1: enables the core performance optimizations in the compiler, while also minimizing compile time.
-                2: provides the best balance between model performance and compile time.
-                3: may provide additional model execution performance but may incur longer compile times and higher host memory usage during model compilation.
+                The level of optimization the compiler should perform. Can be `"1"`, `"2"` or `"3"`, defaults to "2".
+                    1: enables the core performance optimizations in the compiler, while also minimizing compile time.
+                    2: provides the best balance between model performance and compile time.
+                    3: may provide additional model execution performance but may incur longer compile times and higher host memory usage during model compilation.
             subfolder (`str`, defaults to `""`):
                 In case the relevant files are located inside a subfolder of the model repo either locally or on huggingface.co, you can
                 specify the folder name here.
@@ -897,9 +905,13 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
 
         # Get compilation arguments
         auto_cast_type = None if auto_cast is None else auto_cast_type
+        if instance_type is None:
+            instance_type = get_default_compilation_target()
+        instance_type = align_compilation_target(instance_type, override=False)
         compiler_kwargs = {
             "auto_cast": auto_cast,
             "auto_cast_type": auto_cast_type,
+            "instance_type": instance_type,
         }
 
         pipe = TasksManager.get_model_from_task(
@@ -944,6 +956,7 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
                 ip_adapter_args=ip_adapter_args,
                 output_hidden_states=output_hidden_states,
                 torch_dtype=torch_dtype,
+                tensor_parallel_size=tensor_parallel_size,
                 controlnet_ids=controlnet_ids,
                 **input_shapes_copy,
             )
@@ -953,7 +966,7 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
             for name, (model, neuron_config) in models_and_neuron_configs.items():
                 if "vae" in name:  # vae configs are not cached.
                     continue
-                model_config = model.config
+                model_config = getattr(model, "config", None) or neuron_config._config
                 if isinstance(model_config, FrozenDict):
                     model_config = OrderedDict(model_config)
                     model_config = DiffusersPretrainedConfig.from_dict(model_config)
@@ -962,10 +975,10 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
                     config=model_config,
                     input_shapes=neuron_config.input_shapes,
                     compiler_kwargs=compiler_kwargs,
-                    input_names=neuron_config.inputs,
-                    output_names=neuron_config.outputs,
+                    int_dtype=neuron_config.int_dtype,
+                    float_dtype=neuron_config.float_dtype,
                     dynamic_batch_size=neuron_config.dynamic_batch_size,
-                    tensor_parallel_size=tensor_parallel_size,
+                    tensor_parallel_size=neuron_config.tensor_parallel_size,
                     compiler_type=NEURON_COMPILER_TYPE,
                     compiler_version=NEURON_COMPILER_VERSION,
                     inline_weights_to_neff=inline_weights_to_neff,
@@ -987,6 +1000,9 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
 
         if cache_exist:
             # load cache
+            logger.info(
+                f"Neuron cache found at {model_cache_dir}. If you want to recompile the model, please set `disable_neuron_cache=True`."
+            )
             neuron_model = cls.from_pretrained(model_cache_dir, data_parallel_mode=data_parallel_mode)
             # replace weights
             if not inline_weights_to_neff:
@@ -1025,13 +1041,19 @@ class NeuronDiffusionPipelineBase(NeuronTracedModel):
                 library_name=cls.library_name,
                 **input_shapes,
             )
-
-        return cls._from_pretrained(
-            model_id=save_dir_path,
-            config=config,
-            model_save_dir=save_dir,
-            data_parallel_mode=data_parallel_mode,
-        )
+        if get_neuron_major() == -1:
+            logger.warning(
+                "No Neuron device detected! Model loading is skipped as it requires Neuron hardware."
+                "The model compilation was successful and the artifacts were saved."
+            )
+            return None
+        else:
+            return cls._from_pretrained(
+                model_id=save_dir_path,
+                config=config,
+                model_save_dir=save_dir,
+                data_parallel_mode=data_parallel_mode,
+            )
 
     @classmethod
     def _load_config(cls, config_name_or_path: str | os.PathLike, **kwargs):
@@ -1299,16 +1321,16 @@ class NeuronModelTransformer(_NeuronDiffusionModelPart):
         encoder_attention_mask: torch.Tensor | None = None,
         return_dict: bool = True,
     ):
+        hidden_states = hidden_states.to(self.neuron_config.float_dtype)
+        timestep = timestep.to(self.neuron_config.float_dtype)
         if self.neuron_config.MODEL_TYPE == "flux-transformer-2d":
             ids = torch.cat((txt_ids, img_ids), dim=0)
-            timestep = timestep.to(hidden_states.dtype)
-            image_rotary_emb = torch.stack(self.pos_embed(ids), dim=2).to(hidden_states.dtype)
+            image_rotary_emb = torch.stack(self.pos_embed(ids), dim=2).to(self.neuron_config.float_dtype)
             inputs = (hidden_states, encoder_hidden_states, pooled_projections, timestep, image_rotary_emb)
             if guidance is not None:
-                guidance = guidance.to(hidden_states.dtype)
+                guidance = guidance.to(self.neuron_config.float_dtype)
                 inputs += (guidance,)
         elif self.neuron_config.MODEL_TYPE == "pixart-transformer-2d":
-            timestep = timestep.to(hidden_states.dtype)
             inputs = (hidden_states, encoder_hidden_states, timestep, encoder_attention_mask)
 
         outputs = self.model(*inputs)
@@ -1318,6 +1340,11 @@ class NeuronModelTransformer(_NeuronDiffusionModelPart):
             outputs = (outputs,)
         return outputs
 
+    # Dummy context manager
+    @contextmanager
+    def cache_context(self, name: str):
+        yield
+
 
 class NeuronModelVaeEncoder(_NeuronDiffusionModelPart):
     def __init__(
@@ -1330,6 +1357,7 @@ class NeuronModelVaeEncoder(_NeuronDiffusionModelPart):
         super().__init__(model, parent_pipeline, config, neuron_config, DIFFUSION_MODEL_VAE_ENCODER_NAME)
 
     def forward(self, sample: torch.Tensor, return_dict: bool = True):
+        sample = sample.to(self.neuron_config.float_dtype)
         inputs = (sample,)
         outputs = self.model(*inputs)
 
@@ -1360,7 +1388,7 @@ class NeuronModelVaeDecoder(_NeuronDiffusionModelPart):
         return_dict: bool = True,
         generator=None,
     ):
-        inputs = (latent_sample,)
+        inputs = (latent_sample.to(self.neuron_config.float_dtype),)
         if image is not None:
             inputs += (image,)
         if mask is not None:
@@ -1603,3 +1631,13 @@ class NeuronStableDiffusionXLControlNetPipeline(
 class NeuronFluxPipeline(NeuronDiffusionPipelineBase, FluxPipeline):
     main_input_name = "prompt"
     auto_model_class = FluxPipeline
+
+
+class NeuronFluxKontextPipeline(NeuronDiffusionPipelineBase, FluxKontextPipeline):
+    main_input_name = "prompt"
+    auto_model_class = FluxKontextPipeline
+
+
+class NeuronFluxInpaintPipeline(NeuronDiffusionPipelineBase, FluxInpaintPipeline):
+    main_input_name = "prompt"
+    auto_model_class = FluxInpaintPipeline
