@@ -23,18 +23,20 @@ from typing import TYPE_CHECKING, Literal
 
 import torch
 from huggingface_hub import HfApi, HfFolder, hf_hub_download
+from optimum.exporters.tasks import TasksManager
+from optimum.modeling_base import OptimizedModel
+from optimum.utils import logging
+from optimum.utils.save_utils import maybe_load_preprocessors
 from transformers import AutoConfig, AutoModel, GenerationMixin
 
 from optimum.exporters.neuron import main_export
 from optimum.exporters.neuron.model_configs import *  # noqa: F403
-from optimum.exporters.tasks import TasksManager
-from optimum.utils import logging
-from optimum.utils.save_utils import maybe_load_preprocessors
 
 from .cache.entries.single_model import SingleModelCacheEntry
 from .cache.hub_cache import create_hub_compile_cache_proxy
 from .modeling_base import NeuronModel
 from .utils import (
+    DTYPE_MAPPER,
     NEURON_FILE_NAME,
     InputShapesArguments,
     check_if_weights_replacable,
@@ -43,6 +45,8 @@ from .utils import (
     store_compilation_config,
 )
 from .utils.import_utils import is_neuronx_available
+from .utils.instance import align_compilation_target, get_default_compilation_target
+from .utils.system import get_neuron_major
 from .utils.version_utils import check_compiler_compatibility, get_neuroncc_version, get_neuronxcc_version
 
 
@@ -65,7 +69,7 @@ if is_neuronx_available():
 logger = logging.get_logger(__name__)
 
 
-class NeuronTracedModel(NeuronModel):
+class NeuronTracedModel(OptimizedModel, NeuronModel):
     """
     Base class running compiled and optimized models on Neuron devices.
 
@@ -101,6 +105,10 @@ class NeuronTracedModel(NeuronModel):
         **kwargs,
     ):
         super().__init__(model, config)
+        if hasattr(model, "device"):
+            self.device = model.device
+        else:
+            self.device = torch.device("cpu")
 
         self.model = model
         self.model_file_name = model_file_name or NEURON_FILE_NAME
@@ -123,6 +131,7 @@ class NeuronTracedModel(NeuronModel):
             device_id (`int`, defaults to 0):
                 Index of NeuronCore to load the traced model to.
         """
+
         if not isinstance(path, Path):
             path = Path(path)
 
@@ -207,11 +216,12 @@ class NeuronTracedModel(NeuronModel):
         inline_weights_to_neff = config.neuron.get("inline_weights_to_neff", False)
 
         preprocessors = None
+        # Handle model loading based on source (local dir vs hub)
         if model_path.is_dir():
-            model = NeuronTracedModel.load_model(model_path / file_name, to_neuron=not inline_weights_to_neff)
+            model_file_path = model_path / file_name
             new_model_save_dir = model_path
         else:
-            model_cache_path = hf_hub_download(
+            model_file_path = hf_hub_download(
                 repo_id=model_id,
                 filename=file_name,
                 subfolder=subfolder,
@@ -221,9 +231,10 @@ class NeuronTracedModel(NeuronModel):
                 force_download=force_download,
                 local_files_only=local_files_only,
             )
+            new_model_save_dir = Path(model_file_path).parent
 
-            model = NeuronTracedModel.load_model(model_cache_path, to_neuron=not inline_weights_to_neff)
-            new_model_save_dir = Path(model_cache_path).parent
+        # Load model
+        model = NeuronTracedModel.load_model(model_file_path, to_neuron=not inline_weights_to_neff)
 
         preprocessors = maybe_load_preprocessors(model_id, subfolder=subfolder)
 
@@ -260,10 +271,12 @@ class NeuronTracedModel(NeuronModel):
         disable_neuron_cache: bool = False,
         inline_weights_to_neff: bool = True,
         optlevel: str = "2",
+        instance_type: Literal["trn1", "inf2", "trn1n", "trn2"] | None = None,
         subfolder: str = "",
         local_files_only: bool = False,
         trust_remote_code: bool = False,
         task: str | None = None,
+        torch_dtype: str | torch.dtype = torch.float32,
         auto_cast: str | None = None,
         auto_cast_type: str | None = None,
         disable_fast_relayout: bool | None = False,
@@ -289,21 +302,27 @@ class NeuronTracedModel(NeuronModel):
             kwargs_shapes["batch_size"] = 1
             disable_fallback = True  # Turn off the fallback for neuron, otherwise dynamic batching will still fail
         auto_cast_type = None if auto_cast is None else auto_cast_type
+        if instance_type is None:
+            instance_type = get_default_compilation_target()
+        instance_type = align_compilation_target(instance_type, override=False)
         compiler_kwargs = {
             "auto_cast": auto_cast,
             "auto_cast_type": auto_cast_type,
+            "instance_type": instance_type,
+            # Inf1 specific compiler args
             "disable_fast_relayout": disable_fast_relayout,
             "disable_fallback": disable_fallback,
         }
         # clean shapes
         commit_hash = kwargs_shapes.pop("_commit_hash", None)
 
-        if not disable_neuron_cache and is_neuronx_available():  # TODO: support caching of Inf1 as well
+        if not disable_neuron_cache and is_neuronx_available():
             # Check if the cache exists
             compilation_config = store_compilation_config(
                 config=config,
                 input_shapes=kwargs_shapes,
                 compiler_kwargs=compiler_kwargs,
+                float_dtype=torch_dtype,
                 dynamic_batch_size=dynamic_batch_size,
                 tensor_parallel_size=tensor_parallel_size,
                 compiler_type=NEURON_COMPILER_TYPE,
@@ -334,6 +353,7 @@ class NeuronTracedModel(NeuronModel):
                     cache_dir=cache_dir,
                     token=token,
                     framework="pt",
+                    torch_dtype=torch_dtype,
                     local_files_only=local_files_only,
                     force_download=force_download,
                     trust_remote_code=trust_remote_code,
@@ -358,6 +378,7 @@ class NeuronTracedModel(NeuronModel):
                 model_name_or_path=model_id,
                 output=save_dir_path,
                 compiler_kwargs=compiler_kwargs,
+                torch_dtype=torch_dtype,
                 task=task,
                 dynamic_batch_size=dynamic_batch_size,
                 cache_dir=cache_dir,
@@ -377,7 +398,14 @@ class NeuronTracedModel(NeuronModel):
             )
             config = AutoConfig.from_pretrained(save_dir_path)
 
-        return cls._from_pretrained(save_dir_path, config, model_save_dir=save_dir)
+        if get_neuron_major() == -1:
+            logger.warning(
+                "No Neuron device detected! Model loading is skipped as it requires Neuron hardware."
+                "The model compilation was successful and the artifacts were saved."
+            )
+            return None
+        else:
+            return cls._from_pretrained(save_dir_path, config, model_save_dir=save_dir)
 
     def push_to_hub(
         self,
@@ -481,6 +509,8 @@ class NeuronTracedModel(NeuronModel):
             tensor_parallel_size=tensor_parallel_size,
             input_shapes=compile_shapes,
             output_hidden_states=neuron_config.get("output_hidden_states", False),
+            int_dtype=DTYPE_MAPPER.pt(neuron_config.get("int_dtype", "int64")),
+            float_dtype=DTYPE_MAPPER.pt(neuron_config.get("float_dtype", "fp32")),
         )
 
     @classmethod
