@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import copy
+from abc import ABC, abstractmethod
 from typing import Any
 
 import torch
@@ -28,7 +29,7 @@ from .sampling import (
 )
 
 
-class NxDGenerationMixin(GenerationMixin):
+class NxDGenerationMixin(GenerationMixin, ABC):
     """A generation Mixin that can be used to extend NxDPreTrainedModel based classes"""
 
     # These are expected to be set by the GenerationMixin code
@@ -258,17 +259,11 @@ class NxDGenerationMixin(GenerationMixin):
         if "attention_mask" in model_kwargs:
             attention_mask = model_kwargs["attention_mask"]
             if is_for_token_generation:
-                if self.neuron_config.padding_side == "left":
-                    attention_mask = torch.cat(
-                        [attention_mask, attention_mask.new_ones((attention_mask.shape[0], 1))],
-                        dim=-1,
-                    )
-                    attention_mask = attention_mask[:, 1:]
-                else:
-                    attention_mask = torch.cat(
-                        [attention_mask.new_ones((attention_mask.shape[0], 1)), attention_mask],
-                        dim=-1,
-                    )
+                # Prepend 1 to the attention mask (inputs are right padded)
+                attention_mask = torch.cat(
+                    [attention_mask.new_ones((attention_mask.shape[0], 1)), attention_mask],
+                    dim=-1,
+                )
             model_kwargs["attention_mask"] = attention_mask
         return model_kwargs
 
@@ -431,3 +426,7 @@ class NxDGenerationMixin(GenerationMixin):
         """
         # We dont want HF to move parameters to device
         return torch.device("cpu")
+
+    @abstractmethod
+    def reset(self):
+        raise SystemError(f"The reset method must be implemented by {self.__class__.__name__}")
