@@ -37,9 +37,9 @@ from ..models.training import (
     specialize_transformation_specs_for_model,
     to_original_peft_config_for_model,
 )
+from ..models.training.training_utils import _get_model_param_count
 from ..utils.import_utils import is_peft_available
 from ..utils.patching import Patcher
-from ..utils.training_utils import _get_model_param_count
 from .utils.save_and_load import get_peft_model_state_dict
 
 
@@ -48,6 +48,7 @@ if is_peft_available():
     from peft.mapping import PEFT_TYPE_TO_CONFIG_MAPPING, PEFT_TYPE_TO_PREFIX_MAPPING
     from peft.tuners import XLoraModel
     from peft.utils import (
+        PeftType,
         load_peft_weights,
         set_peft_model_state_dict,
     )
@@ -63,6 +64,9 @@ else:
     class XLoraModel:
         pass
 
+    class PeftType:
+        pass
+
     def load_peft_weights(*args, **kwargs):
         pass
 
@@ -110,6 +114,12 @@ class NeuronPeftModel(PeftModel):
         # added the `default` adapter.
         self.recompute_parameters_for_current_stage()
 
+    def get_base_model_prefix(self) -> str:
+        if self.active_peft_config.is_prompt_learning or self.peft_type == PeftType.POLY:
+            return "base_model.model.model"
+        else:
+            return "base_model.model"
+
     def recompute_parameters_for_current_stage(self):
         self._parameter_names_for_current_pp_rank = get_pipeline_parameters_for_current_stage(self)
 
@@ -223,6 +233,11 @@ class NeuronPeftModel(PeftModel):
                     )
 
             # Save the adapter weights.
+            if self.trn_config.async_save:
+                from ..models.training.modeling_utils import _wait_previous_async_save
+
+                _wait_previous_async_save(Path(output_dir) / ADAPTER_MODEL_PARALLEL_SHARDS_DIR_NAME)
+
             neuronx_distributed.trainer.save_checkpoint(
                 output_dir,
                 tag=ADAPTER_MODEL_PARALLEL_SHARDS_DIR_NAME,
@@ -420,6 +435,8 @@ class NeuronPeftModel(PeftModel):
 
         # Filter missing keys specific to the current adapter and tuner prefix.
         for key in load_result.missing_keys:
+            if key not in self.parameters_for_current_stage:
+                continue
             if tuner_prefix in key and adapter_name in key:
                 adapter_missing_keys.append(key)
 
