@@ -168,6 +168,7 @@ class NeuronLoraLayer(LoraLayer):
                 input_is_parallel=self.base_layer.input_is_parallel,
                 sequence_parallel_enabled=self.base_layer.sequence_parallel_enabled,
                 sequence_dimension=self.base_layer.sequence_dimension,
+                tensor_model_parallel_group=self.base_layer.tensor_parallel_group,
             )
             self.lora_B[adapter_name] = nn.Linear(r, self.out_features, bias=lora_bias)
         else:
@@ -179,6 +180,7 @@ class NeuronLoraLayer(LoraLayer):
                 gather_output=self.base_layer.gather_output,
                 sequence_parallel_enabled=self.base_layer.sequence_parallel_enabled,
                 sequence_dimension=self.base_layer.sequence_dimension,
+                tensor_model_parallel_group=self.base_layer.tensor_parallel_group,
             )
 
         self.lora_bias[adapter_name] = lora_bias
@@ -263,7 +265,47 @@ class ParallelLinear(nn.Module, NeuronLoraLayer):
     merge = use_peft_instead_of_optimum_neuron(LoraLinear.merge)
     unmerge = use_peft_instead_of_optimum_neuron(LoraLinear.unmerge)
     get_delta_weight = use_peft_instead_of_optimum_neuron(LoraLinear.get_delta_weight)
-    forward = LoraLinear.forward
+
+    def forward(self, x: torch.Tensor, *args: Any, **kwargs: Any) -> torch.Tensor:
+        self._check_forward_args(x, *args, **kwargs)
+        adapter_names = kwargs.pop("adapter_names", None)
+
+        if adapter_names is not None:
+            raise ValueError("Mixed adapter inference is not supported with optimum-neuron.")
+
+        if self.disable_adapters:
+            if self.merged:
+                self.unmerge()
+            result = self.base_layer(x, *args, **kwargs)
+        elif self.merged:
+            result = self.base_layer(x, *args, **kwargs)
+        else:
+            result = self.base_layer(x, *args, **kwargs)
+            torch_result_dtype = result.dtype
+
+            lora_A_keys = self.lora_A.keys()
+            for active_adapter in self.active_adapters:
+                if active_adapter not in lora_A_keys:
+                    continue
+
+                lora_A = self.lora_A[active_adapter]
+                lora_B = self.lora_B[active_adapter]
+                dropout = self.lora_dropout[active_adapter]
+                scaling = self.scaling[active_adapter]
+                x = self._cast_input_dtype(x, lora_A.weight.dtype)
+                if active_adapter not in self.lora_variant:  # vanilla LoRA
+                    result = result + lora_B(lora_A(dropout(x))) * scaling
+                else:
+                    result = self.lora_variant[active_adapter].forward(
+                        self,
+                        active_adapter=active_adapter,
+                        x=x,
+                        result=result,
+                    )
+
+            result = result.to(torch_result_dtype)
+
+        return result
 
     def __repr__(self):
         rep = super().__repr__()
